# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/002_ProducerLoop.ipynb.

# %% auto 0
__all__ = ['logger']

# %% ../../nbs/002_ProducerLoop.ipynb 1
from .logger import get_logger

# %% ../../nbs/002_ProducerLoop.ipynb 5
logger = get_logger(__name__)
